#include <XSUtil/FunctionUtils/xsFortran.h>
#include <XSUtil/Utils/XSutility.h>
#include <cfortran.h>

PROTOCCALLSFSUB20(SMDEM1I,smdem1i,FLOATV,INT,FLOATV,FLOAT,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,INT,INT,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV)
#define SMDEM1I(ear,ne,abun,z,ninputt,inputt,dem,tval,efil,tabs,nmtval,nin,ncomp,tssm,sum,start,end,fstart,fend,photar) \
           CCALLSFSUB20(SMDEM1I,smdem1i,FLOATV,INT,FLOATV,FLOAT,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,INT,INT,INT,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV,FLOATV, \
           ear,ne,abun,z,ninputt,inputt,dem,tval,efil,tabs,nmtval,nin,ncomp,tssm,sum,start,end,fstart,fend,photar)

void ldpfil(int itype, float** tval, int* nmtval, float** efil, int* nin,
                float** tabs, int* ncomp, int* status);


void smdem1(int itype, float* ear, int ne, float* abun, float dens, float z,
                int ninputt, float* inputt, float* dem, int ifl, float* photar,
                int* status)
{

   // Subroutine to calculate the summed emission from plasma with a 
   // DEM given by the input array. This routine handles the case of 
   // the model being interpolated 
   // Arguments : 
   //       itype     I       i: type of plasma emission file 
   //                                 2 = Mekal 
   //                                 3 = Meka 
   //       ear       R       i: model energy ranges 
   //       ne        I       i: number of model energies 
   //       abun      R       i: abundances 
   //       dens      R       i: density (cm^-3) 
   //       z         R       i: redshift 
   //       ninputt   I       i: number of temperatures 
   //       inputt    R       i: temperatures 
   //       dem       R       i: emission measures for input temperatures 
   //       ifl       I       i: dataset number (unused at present) 
   //       photar    R       r: spectrum 
   //       status    I       r: 0==OK

   // Translated from smdem1.f Feb. 2009 (C.Gordon)
   
   
   // Pointers to local arrays
   //    tval        Tabulated temperatures
   //    efil        Tabulated energies
   //    start       Work array for inibin/erebin
   //    end         Work array for inibin/erebin
   //    fstart      Work array for inibin/erebin
   //    fend        Work array for inibin/erebin
   //    tabs        Tabulated spectra
   //    tssm        Interpolated and summed tabular spectrum
   //    sum         Work array for total summed spectrum

   using namespace XSutility;
   
   *status = 0;
   static bool isInit = true;
   static int itsave = -1;
   static int nesave = 0;
   
   // Check whether we need to reinitialize. If so this is either the first
   // time through or we have switched between plasma emission models.

   if (itype != itsave)
   {
      isInit = true;
      itsave = itype;
   }

   // If the first time through then load the model file. If this file has
   // already been loaded then LDPFIL just returns the pointers to the 
   // appropriate memory.
   
   // LDPFIL owns the memory in these arrays
   static float* tval=0; 
   static float* efil=0;
   static float* tabs=0;
   static int nmtval=0;
   static int nin=0;
   static int ncomp=0;
   
   static auto_array_ptr<float> apTssm(0);
   static auto_array_ptr<float> apSum(0);
   
   if (isInit)
   {
      ldpfil(itype, &tval, &nmtval, &efil, &nin, &tabs, &ncomp, status);
      if (*status)
         return;
      
      if (itype == 1 && ncomp != 13)
      {
         string err("R-S file read does not have 12 elements");
         xs_write(const_cast<char*>(err.c_str()), 5);
         return;
      }
      if (itype == 2 && ncomp != 15)
      {
         string err("Mekal file read does not have 14 elements");
         xs_write(const_cast<char*>(err.c_str()),5);
      }

      // get the memory for the work array which accumulates the spectrum 
      // for a given temperature

      float* oldF = apTssm.reset(new float[nin]);
      delete [] oldF;
      
      // get the memory for the work array which accumulates the 
      // total spectrum

      oldF = apSum.reset(new float[nin]);
      delete [] oldF;
   }
   
   // allocate or reallocate memory for inibin/erebin work arrays

   static auto_array_ptr<float> apStart(0);
   static auto_array_ptr<float> apEnd(0);
   static auto_array_ptr<float> apFstart(0);
   static auto_array_ptr<float> apFend(0);
   if (isInit || ne != nesave)
   {
      float* oldF = apStart.reset(new float[ne]);
      delete [] oldF;
      oldF = apEnd.reset(new float[ne]);
      delete [] oldF;
      oldF = apFstart.reset(new float[ne]);
      delete [] oldF;
      oldF = apFend.reset(new float[ne]);
      delete [] oldF;
      nesave = ne;
   }   
   
   SMDEM1I(ear, ne, abun, z, ninputt, inputt, dem, tval, efil, tabs, 
                nmtval, nin, ncomp, apTssm.get(), apSum.get(), 
                apStart.get(), apEnd.get(),apFstart.get(), 
                apFend.get(), photar);
   
   isInit = false;
}
